/****************************************************************************
** Copyright (C) 2020 MikroElektronika d.o.o.
** Contact: https://www.mikroe.com/contact
**
** Permission is hereby granted, free of charge, to any person obtaining a copy
** of this software and associated documentation files (the "Software"), to deal
** in the Software without restriction, including without limitation the rights
** to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
** copies of the Software, and to permit persons to whom the Software is
** furnished to do so, subject to the following conditions:
** The above copyright notice and this permission notice shall be
** included in all copies or substantial portions of the Software.
**
** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
** DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
**  USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

/*!
 * @file environment2.c
 * @brief Environment 2 Click Driver.
 */

#include "environment2.h"

// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

/**
 * @brief Environment 2 duration of raw data measurement delay.
 * @details Reset delay for 30 milliseconds.
 * @return Nothing.
 *
 * @note None.
 *
 * @endcode
 */
static void dev_measurement_duration_raw_delay ( void );

/**
 * @brief Environment 2 duration of self-test delay.
 * @details Reset delay for 250 milliseconds.
 * @return Nothing.
 *
 * @note None.
 *
 * @endcode
 */
static void dev_measurement_duration_test_delay ( void );

/**
 * @brief Environment 2 duration of temperature and relative humidity data measurement delay.
 * @details Reset delay for 10 milliseconds.
 * @return Nothing.
 *
 * @note None.
 *
 * @endcode
 */
static void dev_measurement_duration_temp_hum_delay ( void );

/**
 * @brief Environment 2 CRC checksum calculation.
 * @details Generated by the CRC algorithm according to the 
 * properties as stated.
 * @return CRC checksum.
 *
 * @note None.
 *
 * @endcode
 */
static uint8_t dev_calc_crc ( uint8_t data_0, uint8_t data_1 );

/**
 * @brief Environment 2 set device slave address.
 * @details This function sets I2C address of the subordinate I2C device.
 * @param[in] ctx : Click context object.
 * See #environment2_t object definition for detailed explanation.
 * @param[in] select_device : 
 *         @li @c 0x00 ( ENVIRONMENT2_SEL_SGP40 ) - Sets I2C address of the SGP40 Sensor,
 *         @li @c 0x01 ( ENVIRONMENT2_SEL_SHT40 ) - Sets I2C address of the SHT40 Sensor.
 * @return @li @c  0 - Success,
 *         @li @c -1 - Error.
 *
 * See #err_t definition for detailed explanation.
 * @note None.
 *
 * @endcode
 */
static err_t dev_set_device_slave_address ( environment2_t *ctx, uint8_t select_device );

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void environment2_cfg_setup ( environment2_cfg_t *cfg ) {
    // Communication gpio pins

    cfg->scl  = HAL_PIN_NC;
    cfg->sda  = HAL_PIN_NC;

    cfg->i2c_speed   = I2C_MASTER_SPEED_STANDARD;
    cfg->i2c_address = ENVIRONMENT2_SGP40_SET_DEV_ADDR;
}

err_t environment2_init ( environment2_t *ctx, environment2_cfg_t *cfg ) {
    i2c_master_config_t i2c_cfg;

    i2c_master_configure_default( &i2c_cfg );

    i2c_cfg.scl = cfg->scl;
    i2c_cfg.sda = cfg->sda;

    ctx->slave_address = cfg->i2c_address;

    if ( i2c_master_open( &ctx->i2c, &i2c_cfg ) == I2C_MASTER_ERROR ) {
        return I2C_MASTER_ERROR;
    }

    if ( i2c_master_set_slave_address( &ctx->i2c, ctx->slave_address ) == I2C_MASTER_ERROR ) {
        return I2C_MASTER_ERROR;
    }

    if ( i2c_master_set_speed( &ctx->i2c, cfg->i2c_speed ) == I2C_MASTER_ERROR ) {
        return I2C_MASTER_ERROR;
    }

    return I2C_MASTER_SUCCESS;
}

err_t environment2_generic_write ( environment2_t *ctx, uint8_t select_device, uint16_t cmd, uint8_t *tx_buf ) {
    uint8_t tx_buf[ 2 ];
    uint8_t cnt;
    err_t status;
    
    status = ENVIRONMENT2_ERROR;
    
    if ( select_device == ENVIRONMENT2_SEL_SGP40 ) {
        if ( dev_set_device_slave_address( ctx, ENVIRONMENT2_SEL_SGP40 ) == ENVIRONMENT2_OK ) {
            tx_buf[ 0 ] = ( uint8_t ) ( cmd >> 8 );
            tx_buf[ 1 ] = ( uint8_t ) cmd;
            
            i2c_master_write( &ctx->i2c, tx_buf, 2 );
            
            status = ENVIRONMENT2_OK;
        }   
    }

    return status;
}

err_t environment2_generic_read ( environment2_t *ctx, uint8_t select_device, uint16_t cmd, uint8_t *rx_buf ) {
    uint8_t tx_buf[ 2 ];
    uint8_t rx_buf[ 6 ];
    err_t status;
    
    status = ENVIRONMENT2_ERROR;
    
    if ( select_device == ENVIRONMENT2_SEL_SGP40 ) {
        tx_buf[ 0 ] = ( uint8_t ) ( cmd >> 8 );
        tx_buf[ 1 ] = ( uint8_t ) cmd;
        
        i2c_master_write( &ctx->i2c, tx_buf, 2 );
        dev_measurement_duration_test_delay( );
        i2c_master_read( &ctx->i2c, rx_buf, 3 ); 
        
        status = ENVIRONMENT2_OK;
    } else if ( select_device == ENVIRONMENT2_SEL_SHT40 ) {
        tx_buf[ 0 ] = ( uint8_t ) cmd;
        
        i2c_master_write( &ctx->i2c, tx_buf, 1 );
        dev_measurement_duration_temp_hum_delay( );
        i2c_master_read( &ctx->i2c, rx_buf, 6 ); 
        
        status = ENVIRONMENT2_OK;
    }
    
    return status;
}

err_t environment2_get_temp_hum ( environment2_t *ctx, float *humidity, float *temperature ) {
    uint8_t tx_buf[ 1 ];
    uint8_t rx_buf[ 6 ];
    uint16_t t_ticks;
    uint16_t rh_ticks;
    err_t status;
    
    status = ENVIRONMENT2_OK;

    tx_buf[ 0 ] = ENVIRONMENT2_SHT40_CMD_MEASURE_T_RH_HIGH_PRECISION;
    
    if ( dev_set_device_slave_address( ctx, ENVIRONMENT2_SEL_SHT40 ) == ENVIRONMENT2_OK ) {
        i2c_master_write( &ctx->i2c, tx_buf, 1 );
        dev_measurement_duration_temp_hum_delay( );
        i2c_master_read( &ctx->i2c, rx_buf, 6 );

        t_ticks = rx_buf[ 0 ];
        t_ticks <<= 8;
        t_ticks |= rx_buf[ 1 ];
    
        rh_ticks = rx_buf[ 3 ];
        rh_ticks <<= 8;
        rh_ticks |= rx_buf[ 4 ];

        *temperature = ( 175 * ( ( ( float ) t_ticks ) / 65535 ) ) - 45;
        *humidity = ( 125 * ( ( ( float ) rh_ticks ) / 65535 ) ) - 6;    
    } else {
        status = ENVIRONMENT2_ERROR;
    }
    
    return status;
}

err_t environment2_get_air_quality ( environment2_t *ctx, uint16_t *air_quality ) {
    uint8_t tx_buf[ 8 ];
    uint8_t rx_buf[ 3 ];
    uint16_t tmp_hum;
    uint16_t tmp_temp;
    float temperature;
    float humidity;
    uint16_t result;
    err_t status;
    
    status = ENVIRONMENT2_OK;
    
    environment2_get_temp_hum ( ctx, &humidity, &temperature );

    tx_buf[ 0 ] = ( uint8_t ) ( ENVIRONMENT2_SGP40_CMD_MEASURE_RAW >> 8 );
    tx_buf[ 1 ] = ( uint8_t ) ENVIRONMENT2_SGP40_CMD_MEASURE_RAW;
    
    tmp_hum = ( uint16_t ) ( humidity * ( 65535.0 / 100.0 ) );
    
    tx_buf[ 2 ] = ( uint8_t ) ( tmp_hum >> 8 );
    tx_buf[ 3 ] = ( uint8_t ) tmp_hum;
    tx_buf[ 4 ] = dev_calc_crc( tx_buf[ 2 ], tx_buf[ 3 ] );
    
    tmp_temp = ( uint16_t ) ( ( temperature + 45.0 ) * ( 65535.0 / 175.0 ) );
    
    tx_buf[ 5 ] = ( uint8_t ) ( tmp_temp >> 8 );
    tx_buf[ 6 ] = ( uint8_t ) tmp_temp;
    tx_buf[ 7 ] = dev_calc_crc( tx_buf[ 5 ], tx_buf[ 6 ] );
    
    if ( dev_set_device_slave_address( ctx, ENVIRONMENT2_SEL_SGP40 ) == ENVIRONMENT2_OK ) {
        i2c_master_write( &ctx->i2c, tx_buf, 8 );
        dev_measurement_duration_raw_delay( );
        i2c_master_read( &ctx->i2c, rx_buf, 3 );

        result = rx_buf[ 0 ];
        result <<= 8;
        result |= rx_buf[ 1 ];
        
        *air_quality = result;
    } else {
        status = ENVIRONMENT2_ERROR;    
    }

    return status;
}

uint16_t environment2_sgp40_measure_test ( environment2_t *ctx ) {
    uint8_t tx_buf[ 2 ];
    uint8_t rx_buf[ 3 ];
    uint16_t result;

    tx_buf[ 0 ] = ( uint8_t ) ( ENVIRONMENT2_SGP40_CMD_MEASURE_TEST >> 8 );
    tx_buf[ 1 ] = ( uint8_t ) ENVIRONMENT2_SGP40_CMD_MEASURE_TEST;
    
    if ( dev_set_device_slave_address( ctx, ENVIRONMENT2_SEL_SGP40 ) == ENVIRONMENT2_OK ) {
        i2c_master_write( &ctx->i2c, tx_buf, 2 );
        dev_measurement_duration_test_delay( );
        i2c_master_read( &ctx->i2c, rx_buf, 3 );

        result = rx_buf[ 0 ];
        result <<= 8;
        result |= rx_buf[ 1 ];    
    }

    return result;
}

err_t environment2_sgp40_heater_off ( environment2_t *ctx ) {
    uint8_t tx_buf[ 2 ];
    err_t status;
    
    status = ENVIRONMENT2_OK;

    tx_buf[ 0 ] = ( uint8_t ) ( ENVIRONMENT2_SGP40_CMD_HEATER_OFF >> 8 );
    tx_buf[ 1 ] = ( uint8_t ) ENVIRONMENT2_SGP40_CMD_HEATER_OFF;
    
    if ( dev_set_device_slave_address( ctx, ENVIRONMENT2_SEL_SGP40 ) == ENVIRONMENT2_OK ) {
        i2c_master_write( &ctx->i2c, tx_buf, 2 );     
    } else {
        status = ENVIRONMENT2_ERROR;
    }
    
    return status;    
}

err_t environment2_sgp40_soft_reset ( environment2_t *ctx ) {
    uint8_t tx_buf[ 2 ];
    err_t status;
    
    status = ENVIRONMENT2_OK;

    tx_buf[ 0 ] = ( uint8_t ) ( ENVIRONMENT2_SGP40_CMD_SOFT_RESET >> 8 );
    tx_buf[ 1 ] = ( uint8_t ) ENVIRONMENT2_SGP40_CMD_SOFT_RESET;
    
    if ( dev_set_device_slave_address( ctx, ENVIRONMENT2_SEL_SGP40 ) == ENVIRONMENT2_OK ) {
        i2c_master_write( &ctx->i2c, tx_buf, 2 );    
    } else {
        status = ENVIRONMENT2_ERROR;
    }
    
    return status; 
}

// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void dev_measurement_duration_raw_delay ( void ) {
    Delay_10ms( );
    Delay_10ms( );
    Delay_10ms( );
}

static void dev_measurement_duration_test_delay ( void ) {
    Delay_100ms( );
    Delay_100ms( );
    Delay_10ms( );
    Delay_10ms( );
    Delay_10ms( );
    Delay_10ms( );
    Delay_10ms( );
}

static void dev_measurement_duration_temp_hum_delay ( void ) {
    Delay_10ms( );    
}

static uint8_t dev_calc_crc ( uint8_t data_0, uint8_t data_1 ) {
    uint8_t i_cnt;
    uint8_t j_cnt;
    uint8_t crc_data[ 2 ];
    uint8_t crc = 0xFF;
    
    crc_data[ 0 ] = data_0;
    crc_data[ 1 ] = data_1;
    
    for ( i_cnt = 0; i_cnt < 2; i_cnt++ ) {
        crc ^= crc_data[ i_cnt ];

        for ( j_cnt = 8; j_cnt > 0; --j_cnt ) {
            if ( crc & 0x80 ) {
                crc = ( crc << 1 ) ^ 0x31u;
            } else {
                crc = ( crc << 1 );
            }
        }
    }

    return crc;
}

static err_t dev_set_device_slave_address ( environment2_t *ctx, uint8_t select_device ) {
    err_t status;
    
    status = ENVIRONMENT2_ERROR;
    
    if ( select_device == ENVIRONMENT2_SEL_SGP40 ) {
        ctx->slave_address = ENVIRONMENT2_SGP40_SET_DEV_ADDR;
        i2c_master_set_slave_address( &ctx->i2c, ctx->slave_address );
         status = ENVIRONMENT2_OK;
    } else if ( select_device == ENVIRONMENT2_SEL_SHT40 ) {
        ctx->slave_address = ENVIRONMENT2_SHT40_SET_DEV_ADDR;
        i2c_master_set_slave_address( &ctx->i2c, ctx->slave_address );
         status = ENVIRONMENT2_OK;
    }
    
    return status; 
}

// ------------------------------------------------------------------------- END
